import type { DirectiveNode, FieldNode, SelectionNode } from "graphql";
import * as graphql from "graphql";
import type { OperationPlan } from "./engine/OperationPlan.js";
import type { __TrackedValueStep } from "./steps/index.js";
/**
 * Given a selection, finds the first directive named `directiveName` and, if
 * found, extracts and returns the value from the argument named
 * `argumentName`, tracking variable access if necessary.
 *
 * @remarks Currently only supports booleans.
 *
 * @internal
 *
 * PERF: inline.
 */
export declare function evalDirectiveArg<T = unknown>(selection: SelectionNode, directiveName: string, argumentName: string, variableValuesStep: __TrackedValueStep, defaultValue: T): T | undefined;
export declare function evalDirectiveArgDirect<T = unknown>(directive: DirectiveNode, argumentName: string, variableValuesStep: __TrackedValueStep, defaultValue: T): T | undefined;
/**
 * @internal
 */
export interface SelectionSetDigest {
    label: string | undefined;
    fields: Map<string, FieldNode[]>;
    deferred: SelectionSetDigest[] | undefined;
}
/**
 * Implements the `GraphQLCollectFields` algorithm - like `CollectFields` the
 * GraphQL spec, but modified such that access to variables is tracked.
 *
 * @see https://spec.graphql.org/draft/#CollectFields()
 *
 * @internal
 */
export declare function graphqlCollectFields(operationPlan: OperationPlan, parentStepId: number, objectType: graphql.GraphQLObjectType, selections: readonly SelectionNode[], isMutation?: boolean, visitedFragments?: {
    [fragmentName: string]: true;
}, selectionSetDigest?: SelectionSetDigest): SelectionSetDigest;
//# sourceMappingURL=graphqlCollectFields.d.ts.map