import type { GraphQLInputObjectType, GraphQLInputType, VariableDefinitionNode } from "graphql";
import { GraphQLList, GraphQLNonNull } from "graphql";
import type { Constraint } from "../constraints.js";
import type { ExecutionExtra, GrafastResultsList, GrafastValuesList } from "../interfaces.js";
import { UnbatchedExecutableStep } from "../step.js";
import type { __ValueStep } from "./__value.js";
import type { AccessStep } from "./access.js";
/**
 * Implements the `__TrackedValueStep(operationPlan, object, constraints, path)`
 * algorithm used to allow runtime AND plan-time access to the three special
 * entities: `variableValues`, `rootValue` and `context`.
 *
 * ExecutableStep-time access can evaluate the `object` passed to the constructor, and
 * will add constraints to the relevant operationPlan.variableValuesConstraints,
 * operationPlan.rootValueConstraints or operationPlan.contextConstraints to
 * ensure that all future variableValues, rootValues and context will match the
 * assumptions made.
 *
 * Run-time access will see the runtime values of these properties, it will
 * **NOT** reference the `object` passed to the constructor.
 *
 * In core this will be used for evaluating `@skip`, `@include`, `@defer` and
 * `@stream` directives so that a different OpPlan will be used if these would
 * change the query plan, but it can also be used within plan resolvers to
 * branch the logic of a plan based on something in these entities.
 */
export declare class __TrackedValueStep<TData = any, TInputType extends GraphQLInputType | ReadonlyArray<VariableDefinitionNode> | undefined = undefined> extends UnbatchedExecutableStep<TData> {
    static $$export: {
        moduleName: string;
        exportName: string;
    };
    isSyncAndSafe: boolean;
    /**
     * Could be anything. In the case of context it could even have exotic
     * entries such as `pgClient`.
     */
    private readonly value;
    /**
     * A reference to the relevant
     * operationPlan.variableValuesConstraints/contextConstraints/rootValueConstraints.
     *
     * @internal
     */
    private readonly constraints;
    /**
     * The path that we are through the original value (the one that
     * `constraints` relates to).
     */
    private readonly path;
    static withGraphQLType<TInputType extends GraphQLInputType, TData = any>(value: TData | undefined, valuePlan: __ValueStep<TData> | AccessStep<TData>, constraints: Constraint[], path: (string | number)[] | undefined, graphqlType: TInputType): __TrackedValueStepWithDollars<TData, TInputType>;
    private nullableGraphQLType;
    private variableDefinitions;
    /**
     * @internal
     */
    constructor(value: TData | undefined, valuePlan: __ValueStep<TData> | AccessStep<TData>, constraints: Constraint[], path?: Array<string | number>, graphqlTypeOrVariableDefinitions?: TInputType);
    execute(_count: number, values: [GrafastValuesList<TData>]): GrafastResultsList<TData>;
    unbatchedExecute(_extra: ExecutionExtra, v: TData): TData;
    private getValuePlan;
    /**
     * Get the named property of an object.
     */
    get<TAttribute extends keyof TData & string>(attrName: TAttribute): __TrackedValueStepWithDollars<TData[TAttribute], TInputType extends GraphQLInputObjectType ? ReturnType<TInputType["getFields"]>[TAttribute]["type"] : undefined>;
    /**
     * Get the entry at the given index in an array.
     */
    at<TIndex extends keyof TData & number>(index: TIndex): __TrackedValueStepWithDollars<TData[TIndex], TInputType extends GraphQLList<infer U> ? U & GraphQLInputType : TInputType extends GraphQLNonNull<GraphQLList<infer U>> ? U & GraphQLInputType : undefined>;
    /**
     * Evaluates the current value, and adds a constraint to the OpPlan to ensure
     * that all future evaluations of this property will always return the same
     * value.
     *
     * **WARNING**: avoid using this where possible, it causes OpPlans to split.
     *
     * **WARNING**: this is the most expensive eval, if you need to eval, prefer evalIs, evalHas, etc instead.
     */
    eval(): TData | undefined;
    /**
     * Evaluates if the current value is equal to this specific value, and adds a
     * constraint to the OpPlan to ensure that all future evaluations of this
     * check will always return the same (boolean) result.
     *
     * Should only be used on scalars.
     *
     * **WARNING**: avoid using this where possible, it causes OpPlans to split.
     */
    evalIs(expectedValue: unknown): boolean;
    evalIsEmpty(): boolean;
    /**
     * Evaluates if the current value is an object with the given key, and adds a
     * constraint to the OpPlan to ensure that all future evaluations of this
     * check will always return the same (boolean) result.
     *
     * **WARNING**: avoid using this where possible, it causes OpPlans to split.
     */
    evalHas(key: string): boolean;
    /**
     * Evaluates the length of the current value (assumed to be an array), and
     * adds a constraint to the OpPlan to ensure that all future values will have
     * the same length.
     *
     * **WARNING**: avoid using this where possible, it causes OpPlans to split.
     */
    evalLength(): number | null;
    optimize(): import("../step.js").ExecutableStep<any>;
}
export type __TrackedValueStepWithDollars<TData = any, TInputType extends GraphQLInputType | undefined = undefined> = __TrackedValueStep<TData, TInputType> & (TInputType extends GraphQLInputObjectType ? {
    [key in keyof ReturnType<TInputType["getFields"]> & string as `$${key}`]: __TrackedValueStepWithDollars<TData extends {
        [k in key]: infer U;
    } ? U : any, ReturnType<TInputType["getFields"]>[key]["type"]>;
} : Record<string, never>);
//# sourceMappingURL=__trackedValue.d.ts.map